"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Deploy = void 0;
const os = require("os");
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const kit_1 = require("@salesforce/kit");
const ts_types_1 = require("@salesforce/ts-types");
const kit_2 = require("@salesforce/kit");
const types_1 = require("@salesforce/source-deploy-retrieve/lib/src/client/types");
const deployCommand_1 = require("../../../deployCommand");
const componentSetBuilder_1 = require("../../../componentSetBuilder");
const deployResultFormatter_1 = require("../../../formatters/deployResultFormatter");
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@salesforce/plugin-source', 'deploy');
class Deploy extends deployCommand_1.DeployCommand {
    constructor() {
        super(...arguments);
        this.lifecycleEventNames = ['predeploy', 'postdeploy'];
        this.isAsync = false;
        this.isRest = false;
    }
    async run() {
        await this.deploy();
        this.resolveSuccess();
        return this.formatResult();
    }
    // There are 3 types of deploys:
    //   1. synchronous - deploy metadata and wait for the deploy to complete.
    //   2. asynchronous - deploy metadata and immediately return.
    //   3. recent validation - deploy metadata that's already been validated by the org
    async deploy() {
        this.isAsync = this.getFlag('wait').quantity === 0;
        this.isRest = await this.isRestDeploy();
        this.log(`*** Deploying with ${this.isRest ? 'REST' : 'SOAP'} API ***`);
        if (this.flags.validateddeployrequestid) {
            this.deployResult = await this.deployRecentValidation();
        }
        else {
            // the deployment involves a component set
            const cs = await componentSetBuilder_1.ComponentSetBuilder.build({
                apiversion: this.getFlag('apiversion'),
                sourcepath: this.getFlag('sourcepath'),
                manifest: this.flags.manifest && {
                    manifestPath: this.getFlag('manifest'),
                    directoryPaths: this.getPackageDirs(),
                },
                metadata: this.flags.metadata && {
                    metadataEntries: this.getFlag('metadata'),
                    directoryPaths: this.getPackageDirs(),
                },
            });
            // fire predeploy event for sync and async deploys
            await this.lifecycle.emit('predeploy', cs.toArray());
            if (this.isAsync) {
                // This is an async deploy.  We just kick off the request.
                throw Error('ASYNC DEPLOYS NOT IMPLEMENTED YET');
            }
            else {
                const deploy = cs.deploy({
                    usernameOrConnection: this.org.getUsername(),
                    apiOptions: {
                        ignoreWarnings: this.getFlag('ignorewarnings', false),
                        rollbackOnError: !this.getFlag('ignoreerrors', false),
                        checkOnly: this.getFlag('checkonly', false),
                        runTests: this.getFlag('runtests'),
                        testLevel: this.getFlag('testlevel'),
                    },
                });
                // if SFDX_USE_PROGRESS_BAR is unset or true (default true) AND we're not print JSON output
                if (kit_2.env.getBoolean('SFDX_USE_PROGRESS_BAR', true) && !this.isJsonOutput()) {
                    this.initProgressBar();
                    this.progress(deploy);
                }
                this.deployResult = await deploy.start();
            }
        }
        await this.lifecycle.emit('postdeploy', this.deployResult);
        const deployId = ts_types_1.getString(this.deployResult, 'response.id');
        if (deployId) {
            this.displayDeployId(deployId);
            const file = this.getStash();
            // TODO: I think we should stash the ID as soon as we know it.
            this.logger.debug(`Stashing deploy ID: ${deployId}`);
            await file.write({ [deployCommand_1.DeployCommand.STASH_KEY]: { jobid: deployId } });
        }
    }
    resolveSuccess() {
        const status = ts_types_1.getString(this.deployResult, 'response.status');
        if (status !== types_1.RequestStatus.Succeeded) {
            this.setExitCode(1);
        }
    }
    formatResult() {
        const formatterOptions = {
            verbose: this.getFlag('verbose', false),
            async: this.isAsync,
        };
        const formatter = new deployResultFormatter_1.DeployResultFormatter(this.logger, this.ux, formatterOptions, this.deployResult);
        // Only display results to console when JSON flag is unset.
        if (!this.isJsonOutput()) {
            formatter.display();
        }
        return formatter.getJson();
    }
    async deployRecentValidation() {
        const conn = this.org.getConnection();
        const id = this.getFlag('validateddeployrequestid');
        // TODO: This is an async call so we need to poll unless `--wait 0`
        //       See mdapiCheckStatusApi.ts for the toolbelt polling impl.
        const response = await conn.deployRecentValidation({ id, rest: this.isRest });
        if (!this.isAsync) {
            // Remove this and add polling if we need to poll in the plugin.
            throw Error('deployRecentValidation polling not yet implemented');
        }
        // This is the deploy ID of the deployRecentValidation response, not
        // the already validated deploy ID (i.e., validateddeployrequestid).
        let validatedDeployId;
        if (ts_types_1.isString(response)) {
            // SOAP API
            validatedDeployId = response;
        }
        else {
            // REST API
            validatedDeployId = response.id;
        }
        return this.report(validatedDeployId);
    }
    progress(deploy) {
        let started = false;
        deploy.onUpdate((data) => {
            // the numCompTot. isn't computed right away, wait to start until we know how many we have
            if (data.numberComponentsTotal && !started) {
                this.displayDeployId(data.id);
                this.progressBar.start(data.numberComponentsTotal + data.numberTestsTotal);
                started = true;
            }
            // the numTestsTot. isn't computed until validated as tests by the server, update the PB once we know
            if (data.numberTestsTotal) {
                this.progressBar.setTotal(data.numberComponentsTotal + data.numberTestsTotal);
            }
            this.progressBar.update(data.numberComponentsDeployed + data.numberTestsCompleted);
        });
        // any thing else should stop the progress bar
        deploy.onFinish((data) => {
            // the final tick of `onUpdate` is actually fired with `onFinish`
            this.progressBar.update(data.response.numberComponentsDeployed + data.response.numberTestsCompleted);
            this.progressBar.stop();
        });
        deploy.onCancel(() => {
            this.progressBar.stop();
        });
        deploy.onError((error) => {
            this.progressBar.stop();
            throw error;
        });
    }
}
exports.Deploy = Deploy;
Deploy.description = messages.getMessage('description');
Deploy.examples = messages.getMessage('examples').split(os.EOL);
Deploy.requiresProject = true;
Deploy.requiresUsername = true;
Deploy.flagsConfig = {
    checkonly: command_1.flags.boolean({
        char: 'c',
        description: messages.getMessage('flags.checkonly'),
    }),
    soapdeploy: command_1.flags.boolean({
        default: false,
        description: messages.getMessage('flags.soapDeploy'),
    }),
    wait: command_1.flags.minutes({
        char: 'w',
        default: kit_1.Duration.minutes(Deploy.DEFAULT_SRC_WAIT_MINUTES),
        min: kit_1.Duration.minutes(0),
        description: messages.getMessage('flags.wait'),
    }),
    testlevel: command_1.flags.enum({
        char: 'l',
        description: messages.getMessage('flags.testLevel'),
        options: ['NoTestRun', 'RunSpecifiedTests', 'RunLocalTests', 'RunAllTestsInOrg'],
        default: 'NoTestRun',
    }),
    runtests: command_1.flags.array({
        char: 'r',
        description: messages.getMessage('flags.runTests'),
        default: [],
    }),
    ignoreerrors: command_1.flags.boolean({
        char: 'o',
        description: messages.getMessage('flags.ignoreErrors'),
    }),
    ignorewarnings: command_1.flags.boolean({
        char: 'g',
        description: messages.getMessage('flags.ignoreWarnings'),
    }),
    validateddeployrequestid: command_1.flags.id({
        char: 'q',
        description: messages.getMessage('flags.validateDeployRequestId'),
        exclusive: [
            'manifest',
            'metadata',
            'sourcepath',
            'checkonly',
            'testlevel',
            'runtests',
            'ignoreerrors',
            'ignorewarnings',
        ],
    }),
    verbose: command_1.flags.builtin({
        description: messages.getMessage('flags.verbose'),
    }),
    metadata: command_1.flags.array({
        char: 'm',
        description: messages.getMessage('flags.metadata'),
        exclusive: ['manifest', 'sourcepath'],
    }),
    sourcepath: command_1.flags.array({
        char: 'p',
        description: messages.getMessage('flags.sourcePath'),
        exclusive: ['manifest', 'metadata'],
    }),
    manifest: command_1.flags.filepath({
        char: 'x',
        description: messages.getMessage('flags.manifest'),
        exclusive: ['metadata', 'sourcepath'],
    }),
};
//# sourceMappingURL=deploy.js.map