"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeployCommand = void 0;
const source_deploy_retrieve_1 = require("@salesforce/source-deploy-retrieve");
const core_1 = require("@salesforce/core");
const ts_types_1 = require("@salesforce/ts-types");
const kit_1 = require("@salesforce/kit");
const sourceCommand_1 = require("./sourceCommand");
class DeployCommand extends sourceCommand_1.SourceCommand {
    constructor() {
        super(...arguments);
        this.deployIdDisplayed = false;
    }
    /**
     * Request a report of an in-progess or completed deployment.
     *
     * @param id the Deploy ID of a deployment request
     * @returns DeployResult
     */
    async report(id) {
        const deployId = this.resolveDeployId(id);
        this.displayDeployId(deployId);
        const res = await this.org.getConnection().metadata.checkDeployStatus(deployId, true);
        const deployStatus = res;
        return new source_deploy_retrieve_1.DeployResult(deployStatus, new source_deploy_retrieve_1.ComponentSet());
    }
    getStash() {
        return new core_1.ConfigFile({ isGlobal: true, filename: 'stash.json' });
    }
    resolveDeployId(id) {
        if (id) {
            return id;
        }
        else {
            // try and read from the ~/.sfdx/stash.json file for the most recent deploy ID
            try {
                this.logger.debug('Reading from ~/.sfdx/stash.json for the deploy id');
                const stash = this.getStash();
                stash.readSync(true);
                return ts_types_1.asString(stash.get(DeployCommand.STASH_KEY).jobid);
            }
            catch (err) {
                const error = err;
                if (error.code === 'ENOENT') {
                    throw core_1.SfdxError.create('@salesforce/plugin-source', 'deploy', 'MissingDeployId');
                }
                throw core_1.SfdxError.wrap(error);
            }
        }
    }
    displayDeployId(id) {
        if (!this.isJsonOutput() && !this.deployIdDisplayed) {
            this.ux.log(`Deploy ID: ${id}`);
            this.deployIdDisplayed = true;
        }
    }
    // REST is the default unless:
    //   1. SOAP is specified with the soapdeploy flag on the command
    //   2. The restDeploy SFDX config setting is explicitly false.
    async isRestDeploy() {
        if (ts_types_1.getBoolean(this.flags, 'soapdeploy') === true) {
            this.logger.debug('soapdeploy flag === true.  Using SOAP');
            return false;
        }
        const aggregator = await core_1.ConfigAggregator.create();
        const restDeployConfig = aggregator.getPropertyValue('restDeploy');
        // aggregator property values are returned as strings
        if (restDeployConfig === 'false') {
            this.logger.debug('restDeploy SFDX config === false.  Using SOAP');
            return false;
        }
        else if (restDeployConfig === 'true') {
            this.logger.debug('restDeploy SFDX config === true.  Using REST');
        }
        else {
            this.logger.debug('soapdeploy flag unset. restDeploy SFDX config unset.  Defaulting to REST');
        }
        return true;
    }
    async poll(deployId, options) {
        var _a, _b;
        const defaultOptions = {
            frequency: (_a = options === null || options === void 0 ? void 0 : options.frequency) !== null && _a !== void 0 ? _a : kit_1.Duration.seconds(1),
            timeout: (_b = options === null || options === void 0 ? void 0 : options.timeout) !== null && _b !== void 0 ? _b : this.flags.wait,
            poll: async () => {
                const deployResult = await this.report(deployId);
                return {
                    completed: ts_types_1.getBoolean(deployResult, 'response.done'),
                    payload: deployResult,
                };
            },
        };
        const pollingOptions = { ...defaultOptions, ...options };
        const pollingClient = await core_1.PollingClient.create(pollingOptions);
        return pollingClient.subscribe();
    }
}
exports.DeployCommand = DeployCommand;
DeployCommand.STASH_KEY = 'SOURCE_DEPLOY';
//# sourceMappingURL=deployCommand.js.map