"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeployResultFormatter = void 0;
const path = require("path");
const chalk = require("chalk");
const core_1 = require("@salesforce/core");
const ts_types_1 = require("@salesforce/ts-types");
const types_1 = require("@salesforce/source-deploy-retrieve/lib/src/client/types");
const resultFormatter_1 = require("./resultFormatter");
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@salesforce/plugin-source', 'deploy');
class DeployResultFormatter extends resultFormatter_1.ResultFormatter {
    constructor(logger, ux, options, result) {
        super(logger, ux, options);
        this.result = result;
        this.fileResponses = (result === null || result === void 0 ? void 0 : result.getFileResponses) ? result.getFileResponses() : [];
    }
    /**
     * Get the JSON output from the DeployResult. The returned JSON shape
     * varies based on:
     *
     * 1. Standard synchronous deploy
     * 2. Asynchronous deploy (wait=0)
     *
     * @returns a JSON formatted result matching the provided type.
     */
    getJson() {
        const json = this.getResponse();
        json.deployedSource = this.fileResponses;
        json.outboundFiles = []; // to match toolbelt version
        json.deploys = [Object.assign({}, this.getResponse())]; // to match toolbelt version
        if (this.isAsync()) {
            // json = this.getResponse(); // <-- TODO: ensure the response matches toolbelt
            return json;
        }
        return json;
    }
    /**
     * Displays deploy results in human format.  Output can vary based on:
     *
     * 1. Standard synchronous deploy (no tests run)
     * 2. Asynchronous deploy (wait=0)
     * 3. Checkonly deploy (checkonly=true)
     * 4. Deploy with test results
     */
    display() {
        // Display check-only, non-verbose success
        if (this.isSuccess() && this.isCheckOnly() && !this.isVerbose()) {
            const componentsDeployed = this.getNumResult('numberComponentsDeployed');
            const testsCompleted = this.getNumResult('numberTestsCompleted');
            const checkOnlySuccessMsg = messages.getMessage('checkOnlySuccess', [componentsDeployed, testsCompleted]);
            this.ux.log(checkOnlySuccessMsg);
            return;
        }
        if (this.hasStatus(types_1.RequestStatus.Canceled)) {
            const canceledByName = ts_types_1.getString(this.result, 'response.canceledByName', 'unknown');
            throw new core_1.SfdxError(messages.getMessage('deployCanceled', [canceledByName]), 'DeployFailed');
        }
        this.displaySuccesses();
        this.displayFailures();
        this.displayTestResults();
        // Throw a DeployFailed error unless the deployment was successful.
        if (!this.isSuccess()) {
            throw new core_1.SfdxError(messages.getMessage('deployFailed'), 'DeployFailed');
        }
    }
    hasStatus(status) {
        return ts_types_1.getString(this.result, 'response.status') === status;
    }
    hasComponents() {
        return ts_types_1.getNumber(this.result, 'components.size', 0) > 0;
    }
    isRunTestsEnabled() {
        return ts_types_1.getBoolean(this.result, 'response.runTestsEnabled', false);
    }
    isCheckOnly() {
        return ts_types_1.getBoolean(this.result, 'response.checkOnly', false);
    }
    getNumResult(field) {
        return ts_types_1.getNumber(this.result, `response.${field}`, 0);
    }
    getResponse() {
        return ts_types_1.get(this.result, 'response', {});
    }
    displaySuccesses() {
        if (this.isSuccess() && this.hasComponents()) {
            //  sort by type then filename then fullname
            const files = this.fileResponses.sort((i, j) => {
                if (i.fullName === j.fullName) {
                    // same metadata type, according to above comment sort on filename
                    if (i.filePath === j.filePath) {
                        // same filename's according to comment sort by fullName
                        return i.fullName < j.fullName ? 1 : -1;
                    }
                    return i.filePath < j.filePath ? 1 : -1;
                }
                return i.type < j.type ? 1 : -1;
            });
            // get relative path for table output
            files.forEach((file) => {
                if (file.filePath) {
                    file.filePath = path.relative(process.cwd(), file.filePath);
                }
            });
            this.ux.log('');
            this.ux.styledHeader(chalk.blue('Deployed Source'));
            this.ux.table(files, {
                columns: [
                    { key: 'fullName', label: 'FULL NAME' },
                    { key: 'type', label: 'TYPE' },
                    { key: 'filePath', label: 'PROJECT PATH' },
                ],
            });
        }
    }
    displayFailures() {
        if (this.hasStatus(types_1.RequestStatus.Failed) && this.hasComponents()) {
            // sort by filename then fullname
            const failures = this.fileResponses
                .filter((fileResponse) => fileResponse.state === 'Failed')
                .sort((i, j) => {
                if (i.filePath === j.filePath) {
                    // if they have the same directoryName then sort by fullName
                    return i.fullName < j.fullName ? 1 : -1;
                }
                return i.filePath < j.filePath ? 1 : -1;
            });
            this.ux.log('');
            this.ux.styledHeader(chalk.red(`Component Failures [${failures.length}]`));
            // TODO: do we really need the project path or file path in the table?
            // Seems like we can just provide the full name and devs will know.
            this.ux.table(failures, {
                columns: [
                    { key: 'problemType', label: 'Type' },
                    { key: 'fullName', label: 'Name' },
                    { key: 'error', label: 'Problem' },
                ],
            });
            this.ux.log('');
        }
    }
    displayTestResults() {
        if (this.isRunTestsEnabled()) {
            this.ux.log('');
            if (this.isVerbose()) {
                this.ux.log('TBD: Show test successes, failures, and code coverage');
            }
            else {
                this.ux.styledHeader(chalk.blue('Test Results Summary'));
                this.ux.log(`Passing: ${this.getNumResult('numberTestsCompleted')}`);
                this.ux.log(`Failing: ${this.getNumResult('numberTestErrors')}`);
                this.ux.log(`Total: ${this.getNumResult('numberTestsTotal')}`);
                this.ux.log(`Time: ${this.getNumResult('details.runTestResult.totalTime')}`);
            }
        }
    }
}
exports.DeployResultFormatter = DeployResultFormatter;
//# sourceMappingURL=deployResultFormatter.js.map