"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.RetrieveResultFormatter = void 0;
const chalk_1 = require("chalk");
const core_1 = require("@salesforce/core");
const ts_types_1 = require("@salesforce/ts-types");
const types_1 = require("@salesforce/source-deploy-retrieve/lib/src/client/types");
const resultFormatter_1 = require("./resultFormatter");
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@salesforce/plugin-source', 'retrieve');
class RetrieveResultFormatter extends resultFormatter_1.ResultFormatter {
    constructor(logger, ux, options, result) {
        super(logger, ux, options);
        this.result = result;
        this.fileResponses = (result === null || result === void 0 ? void 0 : result.getFileResponses) ? result.getFileResponses() : [];
    }
    /**
     * Get the JSON output from the RetrieveResult.
     *
     * @returns RetrieveCommandResult
     */
    getJson() {
        const warnMessages = ts_types_1.get(this.result, 'response.messages', []);
        const warnings = Array.isArray(warnMessages) ? warnMessages : [warnMessages];
        return {
            inboundFiles: this.fileResponses,
            packages: [],
            warnings,
            response: this.result.response,
        };
    }
    /**
     * Displays retrieve results in human format.
     */
    display() {
        if (this.hasStatus(types_1.RequestStatus.InProgress)) {
            const commandWaitTime = ts_types_1.getNumber(this.options, 'waitTime', 33);
            this.ux.log(messages.getMessage('retrieveTimeout', [commandWaitTime]));
            return;
        }
        this.ux.styledHeader(chalk_1.blue(messages.getMessage('retrievedSourceHeader')));
        if (this.isSuccess()) {
            const fileProps = ts_types_1.get(this.result, 'response.fileProperties', []);
            const fileProperties = Array.isArray(fileProps) ? fileProps : [fileProps];
            if (fileProperties.length) {
                const columns = [
                    { key: 'fullName', label: 'FULL NAME' },
                    { key: 'type', label: 'TYPE' },
                    { key: 'fileName', label: 'PROJECT PATH' },
                ];
                this.ux.table(fileProperties, { columns });
            }
            else {
                this.ux.log(messages.getMessage('NoResultsFound'));
            }
        }
        else {
            this.ux.log('Retrieve Failed due to: (check this.result.response.messages)');
        }
        // if (results.status === 'SucceededPartial' && results.successes.length && results.failures.length) {
        //   this.ux.log('');
        //   this.ux.styledHeader(yellow(messages.getMessage('metadataNotFoundWarning')));
        //   results.failures.forEach((warning) => this.ux.log(warning.message));
        // }
        // Display any package retrievals
        // if (results.packages && results.packages.length) {
        //   this.logger.styledHeader(this.logger.color.blue('Retrieved Packages'));
        //   results.packages.forEach(pkg => {
        //     this.logger.log(`${pkg.name} package converted and retrieved to: ${pkg.path}`);
        //   });
        //   this.logger.log('');
        // }
    }
    hasStatus(status) {
        return ts_types_1.getString(this.result, 'response.status') === status;
    }
    hasComponents() {
        return ts_types_1.getNumber(this.result, 'components.size', 0) === 0;
    }
}
exports.RetrieveResultFormatter = RetrieveResultFormatter;
//# sourceMappingURL=retrieveResultFormatter.js.map